pub struct TransitionQueue<T: Behavior> { /* private fields */ }Expand description
A queue of transitions to be invoked automatically.
Implementations
Sourceimpl<T: Behavior> TransitionQueue<T>
impl<T: Behavior> TransitionQueue<T>
Sourcepub fn chain(items: impl IntoIterator<Item = T>) -> Self
pub fn chain(items: impl IntoIterator<Item = T>) -> Self
Creates a new TransitionQueue which starts each given behavior in order.
Unlike TransitionQueue::sequence, a chain does not wait for the behaviors to stop
and starts the behaviors on top of each other. This is useful when you wish to load a set of
states onto the behavior stack (i.e. “Bird must Fly and Chirp”).
You must ensure that each transition in the chain is allowed (see Behavior::filter_next).
The sequence will stop if any transition fails.
Sourcepub fn sequence(items: impl IntoIterator<Item = T>) -> Self
pub fn sequence(items: impl IntoIterator<Item = T>) -> Self
Creates a new TransitionQueue which starts each given behavior in order,
waiting for each one to stop before starting the next.
Unlike TransitionQueue::chain, a sequence waits for each behavior to stop before starting the next.
This is useful when you want to queue some actions after each other (e.g. “Bird must Chirp, then Fly”).
You must ensure each transition from the current behavior to all elements in
the sequence is allowed (see Behavior::filter_next).
The sequence will stop if any transition fails.
Sourcepub fn empty() -> Self
pub fn empty() -> Self
Creates an empty TransitionQueue.
Sourcepub fn start(next: T) -> Self
pub fn start(next: T) -> Self
Creates a new TransitionQueue which starts with the given Behavior.
Sourcepub fn stop() -> Self
pub fn stop() -> Self
Creates a new TransitionQueue which starts by stopping the current Behavior.
Sourcepub fn wait_for(next: T) -> Self
pub fn wait_for(next: T) -> Self
Creates a new TransitionQueue which starts with the given Behavior and waits for it to stop.
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Returns true if the TransitionQueue is empty.
Sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
Returns the number of transitions in the TransitionQueue.
Sourcepub fn then(self, next: T) -> Self
pub fn then(self, next: T) -> Self
Extends the TransitionQueue by starting the next Behavior.
Sourcepub fn then_if(self, condition: bool, next: T) -> Self
pub fn then_if(self, condition: bool, next: T) -> Self
Extends the TransitionQueue by starting the next Behavior if and only if condition is true.
This is just a convenient shortcut useful when creating complex queues, for example:
use bevy::prelude::*;
use moonshine_behavior::prelude::*;
#[derive(Component, Debug)]
pub enum Foo {
Bar,
Baz,
}
impl Behavior for Foo {}
fn make_queue(baz: bool) -> TransitionQueue<Foo> {
TransitionQueue::start(Foo::Bar).then_if(baz, Foo::Baz)
}Sourcepub fn then_wait_for(self, next: T) -> Self
pub fn then_wait_for(self, next: T) -> Self
Extends the TransitionQueue by starting the next Behavior and waiting for it to stop.
Sourcepub fn then_stop(self) -> Self
pub fn then_stop(self) -> Self
Extends the TransitionQueue by stopping the current Behavior.